@@ -12,7 +12,7 @@ DEopt <- function(OF, algo = list(), ...) {
                   loopRepair = TRUE,
                   methodOF = "loop",
                   printDetail = TRUE,
-                  printBar = TRUE,
+                  printBar = interactive(),
                   initP = NULL,
                   storeF = TRUE,
                   storeSolutions = FALSE,
